import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { CreateAdCommand } from './create-ad.command';
import { Inject } from '@nestjs/common';
import {
  AD_REPOSITORY,
  PARAMS_PROVIDER,
  TIMEZONE_FINDER,
  TIME_CONVERTER,
} from '@modules/ad/ad.di-tokens';
import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import { Waypoint } from '../../types/waypoint';
import { DefaultParams } from '../../ports/default-params.type';
import { AdRepositoryPort } from '../../ports/ad.repository.port';
import { DefaultParamsProviderPort } from '../../ports/default-params-provider.port';
import { AdAlreadyExistsException } from '@modules/ad/core/domain/ad.errors';
import { AggregateID, ConflictException } from '@mobicoop/ddd-library';
import { ScheduleItem } from '../../types/schedule-item';
import { TimeConverterPort } from '../../ports/time-converter.port';
import { TimezoneFinderPort } from '../../ports/timezone-finder.port';
import { Frequency } from '@modules/ad/core/domain/ad.types';

@CommandHandler(CreateAdCommand)
export class CreateAdService implements ICommandHandler {
  private readonly _defaultParams: DefaultParams;

  constructor(
    @Inject(AD_REPOSITORY)
    private readonly repository: AdRepositoryPort,
    @Inject(PARAMS_PROVIDER)
    private readonly defaultParamsProvider: DefaultParamsProviderPort,
    @Inject(TIMEZONE_FINDER)
    private readonly timezoneFinder: TimezoneFinderPort,
    @Inject(TIME_CONVERTER)
    private readonly timeConverter: TimeConverterPort,
  ) {
    this._defaultParams = defaultParamsProvider.getParams();
  }

  async execute(command: CreateAdCommand): Promise<AggregateID> {
    const timezone = this.timezoneFinder.timezones(
      command.waypoints[0].lon,
      command.waypoints[0].lat,
      this._defaultParams.DEFAULT_TIMEZONE,
    )[0];
    const ad = AdEntity.create(
      {
        userId: command.userId,
        driver: command.driver,
        passenger: command.passenger,
        frequency: command.frequency,
        fromDate: this.getFromDate(
          command.fromDate,
          command.frequency,
          command.schedule[0].time,
          timezone,
        ),
        toDate: this.getToDate(
          command.fromDate,
          command.toDate,
          command.frequency,
          command.schedule[0].time,
          timezone,
        ),
        schedule: command.schedule.map((scheduleItem: ScheduleItem) => ({
          day: this.getDay(
            scheduleItem.day,
            command.fromDate,
            command.frequency,
            scheduleItem.time,
            timezone,
          ),
          time: this.getTime(
            command.fromDate,
            command.frequency,
            scheduleItem.time,
            timezone,
          ),
          margin: scheduleItem.margin,
        })),
        seatsProposed: command.seatsProposed,
        seatsRequested: command.seatsRequested,
        strict: command.strict,
        waypoints: command.waypoints.map((waypoint: Waypoint) => ({
          position: waypoint.position,
          address: {
            name: waypoint.name,
            houseNumber: waypoint.houseNumber,
            street: waypoint.street,
            postalCode: waypoint.postalCode,
            locality: waypoint.locality,
            country: waypoint.country,
            coordinates: {
              lon: waypoint.lon,
              lat: waypoint.lat,
            },
          },
        })),
      },
      {
        driver: this._defaultParams.DRIVER,
        passenger: this._defaultParams.PASSENGER,
        marginDuration: this._defaultParams.DEPARTURE_TIME_MARGIN,
        strict: this._defaultParams.STRICT,
        seatsProposed: this._defaultParams.SEATS_PROPOSED,
        seatsRequested: this._defaultParams.SEATS_REQUESTED,
      },
    );

    try {
      await this.repository.insert(ad);
      return ad.id;
    } catch (error: any) {
      if (error instanceof ConflictException) {
        throw new AdAlreadyExistsException(error);
      }
      throw error;
    }
  }

  private getFromDate = (
    fromDate: string,
    frequency: Frequency,
    time: string,
    timezone: string,
  ): string => {
    if (frequency === Frequency.RECURRENT) return fromDate;
    return this.timeConverter
      .localStringDateTimeToUtcDate(fromDate, time, timezone)
      .toISOString();
  };

  private getToDate = (
    fromDate: string,
    toDate: string,
    frequency: Frequency,
    time: string,
    timezone: string,
  ): string => {
    if (frequency === Frequency.RECURRENT) return toDate;
    return this.getFromDate(fromDate, frequency, time, timezone);
  };

  private getDay = (
    day: number,
    fromDate: string,
    frequency: Frequency,
    time: string,
    timezone: string,
  ): number => {
    if (frequency === Frequency.RECURRENT)
      return this.getRecurrentDay(day, time, timezone);
    return new Date(
      this.getFromDate(fromDate, frequency, time, timezone),
    ).getDay();
  };

  private getTime = (
    fromDate: string,
    frequency: Frequency,
    time: string,
    timezone: string,
  ): string => {
    if (frequency === Frequency.RECURRENT)
      return this.timeConverter.localStringTimeToUtcStringTime(time, timezone);
    return new Date(
      this.getFromDate(fromDate, frequency, time, timezone),
    ).toTimeString();
  };

  private getRecurrentDay = (
    day: number,
    time: string,
    timezone: string,
  ): number => {
    // continuer ici
    const baseDate = new Date('1970-01-01T00:00:00Z');
    const hour = parseInt(time.split(':')[0]);
    const utcHour = parseInt(
      this.timeConverter
        .localStringTimeToUtcStringTime(time, timezone)
        .split(':')[0],
    );
    if (utcHour >= 11 && hour < 13) return day > 0 ? day - 1 : 6;
    return day;
  };
}
